# -*- coding: utf-8 -*-

from django.contrib.auth.models import Permission
from django.core.exceptions import FieldError
from django.db.models import Q
from django.http import Http404

from rest_framework import generics, viewsets
from rest_framework.response import Response

from . import serializers, exceptions


class QuerysetMixin:
    def get_filters(self):
        filters = dict()
        for key, value in self.request.GET.items():
            if key.startswith('filter__'):
                key = key[8:]
                if key.endswith('isnull'):
                    if value.lower() == 'true':
                        filters[key] = True
                    elif value.lower() == 'false':
                        filters[key] = False
                    else:
                        filters[key] = value
                elif key.endswith('__in'):
                    filters[key] = value.split(",")
                else:
                    filters[key] = value
        return filters

    def get_excludes(self):
        excludes = dict()
        for key, value in self.request.GET.items():
            if key.startswith('exclude__'):
                key = key[9:]
                if key.endswith('isnull'):
                    if value.lower() == 'true':
                        excludes[key] = True
                    elif value.lower() == 'false':
                        excludes[key] = False
                    else:
                        excludes[key] = value
                else:
                    excludes[key] = value
        return excludes

    def get_order_by(self):
        # FIXME DOCS order_by=-date.name
        order_by = list()
        for key, value in self.request.GET.items():
            if key.startswith('order_by'):
                order_by = value.split('.')
        return order_by

    def get_queryset(self):
        filters = self.get_filters()
        excludes = self.get_excludes()
        order_by = self.get_order_by()
        queryset = super(QuerysetMixin, self).get_queryset()
        try:
            if filters:
                queryset = queryset.filter(**filters)
            if excludes:
                for key, value in excludes.items():
                    queryset = queryset.exclude(**{key: value})
            if order_by:
                queryset = queryset.order_by(*order_by)
        except ValueError:
            # just a little philosophy but it may be correct to raise a 400 bad request
            # without information
            raise Http404
        except FieldError:
            # TODO migliorare messaggio di errore
            raise exceptions.BadRequest
        return queryset.distinct()

    @property
    def paginator(self):
        if self.request.query_params.get('no_page'):
            self.pagination_class = None
        return super(QuerysetMixin, self).paginator

    def paginated_list(self, request, custom_qs=None, *args, **kwargs):
        """
        :param custom_qs: non standard queryset (example .values() or .only() Or .defer())
        :return: paginated Response
        """
        if isinstance(custom_qs, type(None)):
            return super().list(request)

        queryset = self.filter_queryset(queryset=custom_qs)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class StatusUpdateViewMixin:
    def put(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.status = self.new_status
        instance.save()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)


class PermissionListView(generics.ListAPIView):
    """
    Get all permissions
    """
    serializer_class = serializers.PermissionSerializer
    pagination_class = None

    def get_queryset(self):
        user = self.request.user
        qs = Permission.objects.all()
        if not user.is_superuser:
            qs = qs.filter(Q(group__user=user) | Q(user=user))

        if 'app_label' in self.kwargs:
            qs = qs.filter(content_type__app_label=self.kwargs['app_label'])
            if 'model_name' in self.kwargs:
                qs = qs.filter(content_type__model=self.kwargs['model_name'])

        return qs


class ThuxListViewMixin(QuerysetMixin):
    pass


class ThuxRetrieveViewMixin:
    pass


class ThuxCreateViewMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None

    # def perform_create(self, serializer):
    #     self.user = self.request.user
    #     serializer.save(
    #         creator=self.user,
    #         last_modifier=self.user,
    #     )


class ThuxUpdateViewMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None

    # def perform_update(self, serializer):
    #     self.user = self.request.user
    #     serializer.save(
    #         last_modifier=self.user
    #     )


class ThuxDestroyViewMixin:
    pass


class ThuxStatusViewMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None

    # def perform_update(self, serializer):
    #     self.user = self.request.user
    #     serializer.save(
    #         last_modifier=self.user,
    #         status=self.new_status
    #     )


class ThuxViewMixin(
    ThuxListViewMixin, ThuxRetrieveViewMixin, ThuxCreateViewMixin, ThuxUpdateViewMixin, ThuxDestroyViewMixin
):
    pass
